/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

#include "debug.h"
#include "ch9431_example.h"

/* ÿļdebugӡĿأ0Խֹļڲӡ */
#define DEBUG_PRINT_IN_THIS_FILE 1
#if DEBUG_PRINT_IN_THIS_FILE
    #define PRINTF(...) printf(__VA_ARGS__)
#else
    #define PRINTF(...) do {} while (0)
#endif

/*
 * ֲʱKeilҪѡ C/C++µC99 ModeGNU extensions
 */
/*
 * CH32V20xCH9431
 *
 * PA4 -- NCS
 * PA5 -- SCK
 * PA6 -- SO
 * PA7 -- SI
 * PB6 -- NTX0RTS
 * PB7 -- NTX1RTS
 * PB8 -- NTX2RTS
 * PB10 -- NINT
 * PB11 -- NRX0IP
 * PB12 -- NRX1IP
*/

volatile uint8_t ch9431_int_sign, ch9431_rx0_sign, ch9431_rx1_sign;

/*********************************************************************
 * @fn      SPI_FullDuplex_Init
 *
 * @brief   Configuring the SPI for full-duplex communication.
 *
 * @return  none
 */
void SPI_FullDuplex_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    EXTI_InitTypeDef EXTI_InitStructure = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};
    SPI_InitTypeDef  SPI_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_2);    /* λrst */

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;

    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;

    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                              /* SPI_CPOL_Low SPI_CPOL_High */
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                            /* SPI_CPHA_1Edge SPI_CPHA_2Edge */
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;                               /* SPI_NSS_Hard SPI_NSS_Soft */
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;      /* 48/4=12M */
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_SSOutputCmd(SPI1, DISABLE);

    SPI_Cmd(SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* GPIOB ----> EXTI_Line10 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource10);
    EXTI_InitStructure.EXTI_Line = EXTI_Line10;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line11 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource11);
    EXTI_InitStructure.EXTI_Line = EXTI_Line11;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line12 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource12);
    EXTI_InitStructure.EXTI_Line = EXTI_Line12;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
    NVIC_Init(&NVIC_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_6);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_7);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_8);
}

uint8_t ch9431_tx0_sign, ch9431_tx1_sign;
uint8_t can_rec_buf0[8], can_rec_buf1[8];
uint8_t can_send_buf[8] = {1, 2, 3, 4, 5, 6, 7, 8};
uint32_t err_time_cnt;

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    rxbndlc_t rxbndlc0, rxbndlc1;
    read_rx_id_t rx_id0, rx_id1;
    rd_status_t rd_status;
    sysstat_t sysstat;

    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);

    printf("SystemClk:%d\r\n", SystemCoreClock);

    printf("CH9431 ECHO Test\r\n");

    SPI_FullDuplex_Init();

    NVIC_DisableIRQ(EXTI15_10_IRQn);

    CH9431_Init();
    ch9431_int_sign = 0;
    ch9431_rx0_sign = 0;
    ch9431_rx1_sign = 0;
    do
    {
        rxipctrl_t rxipctrl = {.B0BFE = 1, .B0BFM = 1, .B1BFE = 1, .B1BFM = 1, .B0BFS = 0, .B1BFS = 0};
        CH9431_WriteByte(R8_RXIPCTRL, rxipctrl.value);
        PRINTF("rxipctrl set ok\n");
    }
    while (0);      /* Ϊж */
    Delay_Ms(10);
    EXTI->INTFR = EXTI_Line10 | EXTI_Line11 | EXTI_Line12;
    NVIC_ClearPendingIRQ(EXTI15_10_IRQn);
    NVIC_EnableIRQ(EXTI15_10_IRQn);

    err_time_cnt = 0;

    while (1)
    {
        if(ch9431_rx0_sign)
        {
            /* RX0IPЧյ׼֡ */
            ch9431_rx0_sign = 0;
            ch9431_tx0_sign = 1;
            if (CH9431_Receive_Buffer0(can_rec_buf0, &rxbndlc0) != 0)
            {
                PRINTF("rec 0, len: %d\n", rxbndlc0.DLC);

                CH9431_Read_RX_ID(CH9431_ReadRX_ID0, &rx_id0);

                PRINTF("SID: %02x\n", rx_id0.rxbn.SID);
                PRINTF("EID: %02x\n", rx_id0.rxbn.EID);
                PRINTF("IDE: %02x\n", rx_id0.rxbn.IDE);
                PRINTF("DLC: %02x\n", rx_id0.rxbndlc.DLC);
                PRINTF("RTR: %02x\n", rx_id0.rxbndlc.RTR);

                if (rx_id0.rxbndlc.RTR == 0)
                {
                    if (rxbndlc0.DLC > 0)
                    {
                        PRINTF("data:");
                        for (uint8_t i = 0; i < rxbndlc0.DLC; i++)
                        {
                            PRINTF(" %02x", can_rec_buf0[i]);
                        }
                        PRINTF("\n");
                    }
                }
                else
                {
                    PRINTF("\n");
                }

            }
            else
            {
                PRINTF("CH9431_Receive_Buffer0 failed\n");
            }
            CH9431_BIT_Modify(R8_SYSINTF, 0x01, 0);                     /* ʹλָжϣֹжϷ */
        }
        else
        {
            if(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_11) == RESET)
            {
                /* ǵ͵ƽ */
                ch9431_rx0_sign = 1;
            }
        }

        if(ch9431_rx1_sign)
        {
            /* RX0IPЧյչ֡ */
            ch9431_rx1_sign = 0;
            ch9431_tx1_sign = 1;
            if (CH9431_Receive_Buffer1(can_rec_buf1, &rxbndlc1) != 0)
            {
                PRINTF("rec 1, len: %d\n", rxbndlc1.DLC);

                CH9431_Read_RX_ID(CH9431_ReadRX_ID1, &rx_id1);
                PRINTF("CID: 0x%x\n", ((rx_id1.rxbn.SID << 18) | (rx_id1.rxbn.EID)));
                PRINTF("SID: %02x\n", rx_id1.rxbn.SID);
                PRINTF("EID: %02x\n", rx_id1.rxbn.EID);
                PRINTF("IDE: %02x\n", rx_id1.rxbn.IDE);
                PRINTF("DLC: %02x\n", rx_id1.rxbndlc.DLC);
                PRINTF("RTR: %02x\n", rx_id1.rxbndlc.RTR);

                if (rx_id1.rxbndlc.RTR == 0)
                {
                    if (rxbndlc1.DLC > 0)
                    {
                        PRINTF("data:");
                        for (uint8_t i = 0; i < rxbndlc1.DLC; i++)
                        {
                            PRINTF(" %02x", can_rec_buf1[i]);
                        }
                        PRINTF("\n");
                    }
                }
                else
                {
                    PRINTF("\n");
                }
            }
            else
            {
                PRINTF("CH9431_Receive_Buffer1 failed\n");
            }
            CH9431_BIT_Modify(R8_SYSINTF, 0x02, 0);                     /* ʹλָжϣֹжϷ */
        }
        else
        {
            if(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_12) == RESET)
            {
                /* ǵ͵ƽ */
                ch9431_rx1_sign = 1;
            }
        }

        if(ch9431_int_sign)
        {
            /* жЧ */
            ch9431_int_sign = 0;
            sysstat.value = CH9431_ReadByte(R8_SYSSTAT);
            PRINTF("ICOD: 0x%02x\n", sysstat.ICOD);
        }
        else
        {
            if(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_10) == RESET)
            {
                /* ǵ͵ƽ */
                ch9431_int_sign = 1;
            }
        }

        if((ch9431_tx0_sign != 0) || (ch9431_tx1_sign!= 0))
        {
            CH9431_Read_STATUS(&rd_status);

            if(ch9431_tx0_sign)
            {
                PRINTF("TX0I:%d\n", rd_status.TX0I);
                if(rd_status.TX0REQ == 0)
                {
                    /* ñķID 0 */
                    do
                    {
                        txbn_t txbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
                        txbn.SID = rx_id0.rxbn.SID + 1;     /* ׼֡ID */
                        txbn.EID = rx_id0.rxbn.EID;         /* չ֡ID */
                        txbn.EXIDE = rx_id0.rxbn.IDE;       /* 1:Ľչʶ,0:Ľͱ׼ʶ */

                        CH9431_WriteByte(R8_TXB0SIDL, txbn.SIDL);
                        CH9431_WriteByte(R8_TXB0SIDH, txbn.SIDH);
                        CH9431_WriteByte(R8_TXB0EIDL, txbn.EIDL);
                        CH9431_WriteByte(R8_TXB0EIDH, txbn.EIDH);
                        PRINTF("txb0 set ok\n");
                    }
                    while (0);
                    CH9431_Send_Buffer0(can_rec_buf0, rx_id0.rxbndlc.DLC, rx_id0.rxbndlc.RTR);
                    ch9431_tx0_sign = 0;
                }
            }

            if(ch9431_tx1_sign)
            {
                PRINTF("TX1I:%d\n", rd_status.TX1I);
                if(rd_status.TX1REQ == 0)
                {
                    /* ñķID 1 */
                    do
                    {
                        txbn_t txbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
                        txbn.SID = rx_id1.rxbn.SID + 2;     /* ׼֡ID */
                        txbn.EID = rx_id1.rxbn.EID;         /* չ֡ID */
                        txbn.EXIDE = rx_id1.rxbn.IDE;       /* 1:Ľչʶ,0:Ľͱ׼ʶ */
                        PRINTF("0x%x, 0x%x, 0x%x\n", ((txbn.SID << 18) | (txbn.EID)), txbn.SID, txbn.EID);
                        CH9431_WriteByte(R8_TXB1SIDL, txbn.SIDL);
                        CH9431_WriteByte(R8_TXB1SIDH, txbn.SIDH);
                        CH9431_WriteByte(R8_TXB1EIDL, txbn.EIDL);
                        CH9431_WriteByte(R8_TXB1EIDH, txbn.EIDH);
                        PRINTF("txb1 set ok\n");
                    }
                    while (0);
                    CH9431_Send_Buffer1(can_rec_buf1, rx_id1.rxbndlc.DLC, rx_id1.rxbndlc.RTR);
                    ch9431_tx1_sign = 0;
                }
            }
        }

        err_time_cnt++;
        if(err_time_cnt > 5000000)
        {
            err_time_cnt = 0;

            do
            {
                eflag_t eflag;
                eflag.value = CH9431_ReadByte(R8_EFLAG);
                PRINTF("eflag:%02x\n", eflag.value);
                if(eflag.RX0OVR)
                {
                    PRINTF("RX0OVR\n");
                    printf("0\n");
                }
                if(eflag.RX1OVR)
                {
                    PRINTF("RX1OVR\n");
                    printf("1\n");
                }
                if(eflag.RX0OVR || eflag.RX1OVR)
                {
                    CH9431_WriteByte(R8_EFLAG, 0);
                }
            }
            while (0);

            do
            {
                uint8_t err_rec, err_tec;

                err_rec = CH9431_ReadByte(R8_REC);
                PRINTF("err_rec:%02x\n", err_rec);

                err_tec = CH9431_ReadByte(R8_TEC);
                PRINTF("err_tec:%02x\n", err_tec);
            }
            while (0);
        }
    }
}
