/********************************** (C) COPYRIGHT *******************************
 * File Name          : ch9431_cmd.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/10/10
 * Description        : This file contains ch9431 command example.
*********************************************************************************
* Copyright (c) 2023 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "ch9431_cmd.h"

/*********************************************************************
 * @fn      CH9431_WriteByte
 *
 * @brief   Write a Byte to a register in CH9431.
 *
 * @param   addr - the addr of register in CH9431.
 * @param   value - the value need to write.
 *
 * @return  None.
 */
void CH9431_WriteByte(uint8_t addr, uint8_t value)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_WRITE);            /* 发送写命令 */

    _port_ch9431_spi_rw_data(addr);                     /* 发送地址 */

    _port_ch9431_spi_rw_data(value);                    /* 写入数据 */

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */
}

/*********************************************************************
 * @fn      CH9431_ReadByte
 *
 * @brief   read a Byte from a register in CH9431.
 *
 * @param   addr - the addr of register in CH9431.
 *
 * @return  value of addr in CH9431.
 */
uint8_t CH9431_ReadByte(uint8_t addr)
{
    uint8_t rByte;

    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_READ);             /* 发送读命令 */

    _port_ch9431_spi_rw_data(addr);                     /* 发送地址 */

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);         /* 等待CH9431处理，准备数据 */

    rByte = _port_ch9431_spi_rw_data(0xff);             /* 读取数据 */

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

    return rByte;
}

/*********************************************************************
 * @fn      CH9431_BIT_Modify
 *
 * @brief   modify some bits from a register in CH9431.
 *
 * @param   addr - the addr of register in CH9431.
 * @param   mask - the bit modify mask.
 * @param   value - the bit modify new value.
 *
 * @return  none.
 */
void CH9431_BIT_Modify(uint8_t addr, uint8_t mask, uint8_t value)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_BIT_MODIFY);       /* 发送BIT_MODIFY命令 */
    _port_ch9431_spi_rw_data(addr);                     /* 发送BIT_MODIFY地址 */
    _port_ch9431_spi_rw_data(mask);                     /* 发送BIT_MODIFY屏蔽字节 */
    _port_ch9431_spi_rw_data(value);                    /* 发送BIT_MODIFY修正字节 */

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

}

/*********************************************************************
 * @fn      CH9431_Read_STATUS
 *
 * @brief   quickly read status from CH9431.
 *
 * @param   pointer of rd_status_t.
 *
 * @return  None.
 */
void CH9431_Read_STATUS(rd_status_t *rd_status)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_RD_STATUS);        /* 发送读RX状态命令 */

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);         /* 等待CH9431处理，准备数据 */

    rd_status->value = _port_ch9431_spi_rw_data(0xff);  /* 读取数据 */

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

}

/*********************************************************************
 * @fn      CH9431_Read_RX_STATUS
 *
 * @brief   quickly read rx status from CH9431.
 *
 * @param   pointer of rx_status_t.
 *
 * @return  None.
 */
void CH9431_Read_RX_STATUS(rx_status_t *rx_status)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_RX_STATUS);        /* 发送读RX状态命令 */

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);         /* 等待CH9431处理，准备数据 */

    rx_status->value = _port_ch9431_spi_rw_data(0xff);  /* 读取数据 */

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

}

/*********************************************************************
 * @fn      CH9431_RTS
 *
 * @brief   request to send of txbn.
 *
 * @param   txb0 - (ENABLE)->=TXB0CTRL.TXREQ = 1.
 * @param   txb1 - (ENABLE)->=TXB1CTRL.TXREQ = 1.
 * @param   txb2 - (ENABLE)->=TXB2CTRL.TXREQ = 1.
 *
 * @return  None.
 */
void CH9431_RTS(FunctionalState txb0, FunctionalState txb1, FunctionalState txb2)
{
    uint8_t cmd = CMD_CAN_RTS;

    _port_ch9431_spi_nss_low();                     /* 置CH9431的CS为低电平 */

    if (txb0 != DISABLE)
    {
        cmd = cmd | 0x01;
    }
    if (txb1 != DISABLE)
    {
        cmd = cmd | 0x02;
    }
    if (txb2 != DISABLE)
    {
        cmd = cmd | 0x04;
    }

    _port_ch9431_spi_rw_data(cmd);                  /* 发送RTS命令 */

    _port_ch9431_spi_nss_high();                    /* 置CH9431的CS为高电平 */

    _port_ch9431_delay_us(CH9431_RTS_BUSY_US);

}

/*********************************************************************
 * @fn      CH9431_LoadTX_ID
 *
 * @brief   quickly load TX data to CH9431.
 *
 * @param   opt - load TX option, CH9431_LoadTX_ID_t.
 * @param   tx_id - pointer to load_tx_id_t.
 *
 * @return  None.
 */
void CH9431_LoadTX_ID(CH9431_LoadTX_ID_t opt, load_tx_id_t *tx_id)
{
    uint8_t *data = tx_id->value;
    uint8_t len = 5;

    _port_ch9431_spi_nss_low();                             /* 置CH9431的CS为低电平 */
    _port_ch9431_spi_rw_data(CMD_CAN_LOAD_TX | opt);        /* 发送load TX命令 */

    while (len != 0)
    {
        _port_ch9431_spi_rw_data(*data);                    /* 写入数据 */
        len--;
        data++;
    }

    _port_ch9431_spi_nss_high();                            /* 置CH9431的CS为高电平 */
}

/*********************************************************************
 * @fn      CH9431_LoadTX_Data
 *
 * @brief   quickly load TX data to CH9431.
 *
 * @param   opt - load TX option, CH9431_LoadTX_ID_t.
 * @param   tx_id - pointer to load_tx_data_t.
 *
 * @return  None.
 */
void CH9431_LoadTX_Data(CH9431_LoadTX_DATA_t opt, load_tx_data_t *tx_data)
{
    uint8_t *data = tx_data->TXBnD;
    uint8_t len = 8;

    _port_ch9431_spi_nss_low();                             /* 置CH9431的CS为低电平 */
    _port_ch9431_spi_rw_data(CMD_CAN_LOAD_TX | opt);        /* 发送load TX命令 */

    while (len != 0)
    {
        _port_ch9431_spi_rw_data(*data);                    /* 写入数据 */
        len--;
        data++;
    }

    _port_ch9431_spi_nss_high();                            /* 置CH9431的CS为高电平 */
}

/*********************************************************************
 * @fn      CH9431_Read_RX_Data
 *
 * @brief   quickly read rx buff from CH9431.
 *
 * @param   opt - read RX option, CH9431_ReadRX_ID_t.
 *          read_rx_data - pointer to read_rx_id_t.
 *
 * @return  None.
 */
void CH9431_Read_RX_ID(CH9431_ReadRX_ID_t opt, read_rx_id_t *read_rx_id)
{
    uint8_t *data = read_rx_id->value;
    uint8_t len = 5;

    _port_ch9431_spi_nss_low();                                 /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_RD_RX_BUFF | opt);         /* 发送读RX状态命令 */

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);                 /* 等待CH9431处理，准备数据 */

    while (len)
    {
        *data = _port_ch9431_spi_rw_data(0xff);                 /* 读取数据 */
        data++;
        len--;
    }

    _port_ch9431_spi_nss_high();                                /* 置CH9431的CS为高电平 */

}

/*********************************************************************
 * @fn      CH9431_Read_RX_Data
 *
 * @brief   quickly read rx buff from CH9431.
 *
 * @param   opt - read RX option, CH9431_ReadRX_DATA_t.
 *          read_rx_data - pointer to read_rx_data_t.
 *
 * @return  None.
 */
void CH9431_Read_RX_Data(CH9431_ReadRX_DATA_t opt, read_rx_data_t *read_rx_data)
{
    uint8_t *data = read_rx_data->RXBnD;
    uint8_t len = 8;

    _port_ch9431_spi_nss_low();                                 /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(CMD_CAN_RD_RX_BUFF | opt);         /* 发送读RX状态命令 */

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);                 /* 等待CH9431处理，准备数据 */

    while (len)
    {
        *data = _port_ch9431_spi_rw_data(0xff);                 /* 读取数据 */
        data++;
        len--;
    }

    _port_ch9431_spi_nss_high();                                /* 置CH9431的CS为高电平 */

}

/*********************************************************************
 * @fn      CH9431_Reset
 *
 * @brief   soft reset CH9431.
 *
 * @param   none.
 *
 * @return  none.
 */
void CH9431_Reset(void)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */
    _port_ch9431_spi_rw_data(CMD_CAN_RESET);            /* 发送软件复位命令 */
    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */
    _port_ch9431_delay_ms(CH9431_START_READY_MS);       /* 复位后，需要延迟等待系统启动 */
}

/*********************************************************************
 * @fn      CH9431_HardReset
 *
 * @brief   hard reset CH9431.
 *
 * @param   none.
 *
 * @return  none.
 */
void CH9431_HardReset(void)
{
    _port_ch9431_rst_low();
    _port_ch9431_delay_ms(1);                           /* 该时间需要根据rst引脚上的电容调整 */
    _port_ch9431_rst_high();
    _port_ch9431_delay_ms(CH9431_START_READY_MS);       /* 复位后，需要延迟等待系统启动 */
}

/*********************************************************************
 * @fn      CH9431_HardReset
 *
 * @brief   hard reset CH9431.
 *
 * @param   none.
 *
 * @return  none.
 */
void CH9431_CS_Wakeup(void)
{
    _port_ch9431_spi_nss_low();
    _port_ch9431_delay_us(CH9431_CS_WAKEUP_US);         /* 该时间需要根据rst引脚上的电容调整 */
    _port_ch9431_spi_nss_high();
    _port_ch9431_delay_ms(CH9431_WAKEUP_READY_MS);      /* 复位后，需要延迟等待系统启动 */
}

/*********************************************************************
 * @fn      CH9431_getVerString
 *
 * @brief   get ver string of CH9431.
 *
 * @param   none.
 *
 * @return  none.
 */
void CH9431_getVerString(uint8_t *ver_str)
{
    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(0);
    _port_ch9431_spi_rw_data(0x76);
    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);         /* 等待CH9431处理，准备数据 */

    ver_str[0] = _port_ch9431_spi_rw_data(0xff);
    ver_str[1] = _port_ch9431_spi_rw_data(0xff);
    ver_str[2] = _port_ch9431_spi_rw_data(0xff);
    ver_str[3] = _port_ch9431_spi_rw_data(0xff);

    ver_str[4] = _port_ch9431_spi_rw_data(0xff);
    ver_str[5] = _port_ch9431_spi_rw_data(0xff);
    ver_str[6] = _port_ch9431_spi_rw_data(0xff);
    ver_str[7] = _port_ch9431_spi_rw_data(0xff);

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

}

#define ADMIN_IAP_ASK_VALUE         0x55aa55aa
#define ADMIN_IAP_CONFIRM_VALUE     0x5a5a5a5a
#define ADMIN_IAP_ACK_VALUE         0xaa55aa55

/*********************************************************************
 * @fn      CH9431_EnterIAP
 *
 * @brief   enter iap mode.
 *
 * @param   none.
 *
 * @return  none.
 */
uint8_t CH9431_EnterIAP(uint32_t rand)
{
    uint8_t read_data[4];
    uint32_t auth_data;

#if 0
    /* 硬件复位 */
    CH9431_HardReset();
#else
    /* 软件复位 */
    CH9431_Reset();              /* 发送复位指令软件复位MCP2515 */
#endif

    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(0);
    _port_ch9431_spi_rw_data(0xaa);
    _port_ch9431_spi_rw_data(0xaa);
    _port_ch9431_spi_rw_data(0x55);
    _port_ch9431_spi_rw_data(0xaa);
    _port_ch9431_spi_rw_data(0x55);

    _port_ch9431_spi_rw_data((uint8_t)(rand));
    _port_ch9431_spi_rw_data((uint8_t)(rand >> 8));
    _port_ch9431_spi_rw_data((uint8_t)(rand >> 16));
    _port_ch9431_spi_rw_data((uint8_t)(rand >> 24));

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

    _port_ch9431_delay_ms(10);                          /* 等待CH9431处理 */

    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(0);
    _port_ch9431_spi_rw_data(0x55);
    _port_ch9431_spi_rw_data(0x55);
    _port_ch9431_spi_rw_data(0xaa);
    _port_ch9431_spi_rw_data(0x55);
    _port_ch9431_spi_rw_data(0xaa);

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);

    read_data[0] = _port_ch9431_spi_rw_data(0xff);
    read_data[1] = _port_ch9431_spi_rw_data(0xff);
    read_data[2] = _port_ch9431_spi_rw_data(0xff);
    read_data[3] = _port_ch9431_spi_rw_data(0xff);

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

    _port_ch9431_delay_ms(1);                           /* 等待系统处理 */

    auth_data = (read_data[3] << 24) | (read_data[2] << 16)
                | (read_data[1] << 8) | (read_data[0]);

    auth_data = auth_data - rand;

    _port_ch9431_spi_nss_low();                         /* 置CH9431的CS为低电平 */

    _port_ch9431_spi_rw_data(0);
    _port_ch9431_spi_rw_data(0xaa);
    _port_ch9431_spi_rw_data(0x5a);
    _port_ch9431_spi_rw_data(0x5a);
    _port_ch9431_spi_rw_data(0x5a);
    _port_ch9431_spi_rw_data(0x5a);

    _port_ch9431_delay_us(CH9431_WAIT_DATA_US);

    _port_ch9431_spi_rw_data((uint8_t)(auth_data));
    _port_ch9431_spi_rw_data((uint8_t)(auth_data >> 8));
    _port_ch9431_spi_rw_data((uint8_t)(auth_data >> 16));
    _port_ch9431_spi_rw_data((uint8_t)(auth_data >> 24));

    _port_ch9431_spi_nss_high();                        /* 置CH9431的CS为高电平 */

    _port_ch9431_delay_ms(10);                          /* 等待CH9431处理 */

    do
    {
        sysstat_t sysstat;

        sysstat.value = CH9431_ReadByte(R8_SYSSTAT);

        if (sysstat.OPMOD != SYSCTRL_REQOP_IAP)
        {
            /* iap mode err */
            return 1;
        }
        else
        {
            /* iap mode ok */
        }
    }
    while (0);

    return 0;
}

