/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

#include "debug.h"
#include "ch9431_example.h"

/* ÿļdebugӡĿأ0Խֹļڲӡ */
#define DEBUG_PRINT_IN_THIS_FILE 1
#if DEBUG_PRINT_IN_THIS_FILE
    #define PRINTF(...) printf(__VA_ARGS__)
#else
    #define PRINTF(...) do {} while (0)
#endif

/*
 * ֲʱKeilҪѡ C/C++µC99 ModeGNU extensions
 */
/*
 * CH32V20xCH9431
 *
 * PA4 -- NCS
 * PA5 -- SCK
 * PA6 -- SO
 * PA7 -- SI
 * PB6 -- NTX0RTS
 * PB7 -- NTX1RTS
 * PB8 -- NTX2RTS
 * PB10 -- NINT
 * PB11 -- NRX0IP
 * PB12 -- NRX1IP
*/

volatile uint8_t ch9431_int_sign, ch9431_rx0_sign, ch9431_rx1_sign;

/*********************************************************************
 * @fn      SPI_FullDuplex_Init
 *
 * @brief   Configuring the SPI for full-duplex communication.
 *
 * @return  none
 */
void SPI_FullDuplex_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    EXTI_InitTypeDef EXTI_InitStructure = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};
    SPI_InitTypeDef  SPI_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_2);    /* λrst */

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;

    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;

    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                              /* SPI_CPOL_Low SPI_CPOL_High */
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                            /* SPI_CPHA_1Edge SPI_CPHA_2Edge */
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;                               /* SPI_NSS_Hard SPI_NSS_Soft */
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;      /* 48/4=12M */
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_SSOutputCmd(SPI1, DISABLE);

    SPI_Cmd(SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* GPIOB ----> EXTI_Line10 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource10);
    EXTI_InitStructure.EXTI_Line = EXTI_Line10;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line11 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource11);
    EXTI_InitStructure.EXTI_Line = EXTI_Line11;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line12 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource12);
    EXTI_InitStructure.EXTI_Line = EXTI_Line12;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
    NVIC_Init(&NVIC_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_6);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_7);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_8);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    printf("SystemClk:%d\r\n", SystemCoreClock);

    printf("CH9431 RXnIP Test\r\n");

    SPI_FullDuplex_Init();

    NVIC_DisableIRQ(EXTI15_10_IRQn);
    CH9431_Init();
    ch9431_int_sign = 0;
    ch9431_rx0_sign = 0;
    ch9431_rx1_sign = 0;
    NVIC_EnableIRQ(EXTI15_10_IRQn);

    do
    {
        rxipctrl_t rxipctrl = {.B0BFE = 1, .B0BFM = 0, .B1BFE = 1, .B1BFM = 0, .B0BFS = 0, .B1BFS = 0};
        CH9431_WriteByte(R8_RXIPCTRL, rxipctrl.value);
        PRINTF("rxipctrl set ok\n");
    } while (0);

    while(1)
    {
        do
        {
            rxipctrl_t rxipctrl;
            rxipctrl.value = CH9431_ReadByte(R8_RXIPCTRL);
            PRINTF("RXIPCTRL:%02x\n", rxipctrl.value);
        } while (0);

        _port_ch9431_delay_ms(100);

        do
        {
            rxipctrl_t rxipctrl_mask = {.B0BFE = 0, .B0BFM = 0, .B1BFE = 0, .B1BFM = 0, .B0BFS = 1, .B1BFS = 1};
            rxipctrl_t rxipctrl = {.B0BFS = 1, .B1BFS = 0};
            CH9431_BIT_Modify(R8_RXIPCTRL, rxipctrl_mask.value, rxipctrl.value);
        } while (0);

        _port_ch9431_delay_ms(400);

        do
        {
            rxipctrl_t rxipctrl;
            rxipctrl.value = CH9431_ReadByte(R8_RXIPCTRL);
            PRINTF("RXIPCTRL:%02x\n", rxipctrl.value);
        } while (0);

        _port_ch9431_delay_ms(100);

        do
        {
            rxipctrl_t rxipctrl_mask = {.B0BFE = 0, .B0BFM = 0, .B1BFE = 0, .B1BFM = 0, .B0BFS = 1, .B1BFS = 1};
            rxipctrl_t rxipctrl = {.B0BFS = 0, .B1BFS = 1};
            CH9431_BIT_Modify(R8_RXIPCTRL, rxipctrl_mask.value, rxipctrl.value);
        } while (0);

        _port_ch9431_delay_ms(400);

    }
}
