/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

#include "debug.h"
#include "ch9431_example.h"

/* ÿļdebugӡĿأ0Խֹļڲӡ */
#define DEBUG_PRINT_IN_THIS_FILE 1
#if DEBUG_PRINT_IN_THIS_FILE
    #define PRINTF(...) printf(__VA_ARGS__)
#else
    #define PRINTF(...) do {} while (0)
#endif

/*
 * ֲʱKeilҪѡ C/C++µC99 ModeGNU extensions
 */
/*
 * CH32V20xCH9431
 *
 * PA4 -- NCS
 * PA5 -- SCK
 * PA6 -- SO
 * PA7 -- SI
 * PB6 -- NTX0RTS
 * PB7 -- NTX1RTS
 * PB8 -- NTX2RTS
 * PB10 -- NINT
 * PB11 -- NRX0IP
 * PB12 -- NRX1IP
*/

volatile uint8_t ch9431_int_sign, ch9431_rx0_sign, ch9431_rx1_sign;

/*********************************************************************
 * @fn      SPI_FullDuplex_Init
 *
 * @brief   Configuring the SPI for full-duplex communication.
 *
 * @return  none
 */
void SPI_FullDuplex_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    EXTI_InitTypeDef EXTI_InitStructure = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};
    SPI_InitTypeDef  SPI_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_2);    /* λrst */

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;

    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;

    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                              /* SPI_CPOL_Low SPI_CPOL_High */
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                            /* SPI_CPHA_1Edge SPI_CPHA_2Edge */
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;                               /* SPI_NSS_Hard SPI_NSS_Soft */
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;      /* 48/4=12M */
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_SSOutputCmd(SPI1, DISABLE);

    SPI_Cmd(SPI1, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* GPIOB ----> EXTI_Line10 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource10);
    EXTI_InitStructure.EXTI_Line = EXTI_Line10;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line11 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource11);
    EXTI_InitStructure.EXTI_Line = EXTI_Line11;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* GPIOB ----> EXTI_Line12 */
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource12);
    EXTI_InitStructure.EXTI_Line = EXTI_Line12;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
    NVIC_Init(&NVIC_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_6);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_7);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, GPIO_Pin_8);
}

uint8_t can_rec_buf0[8], can_rec_buf1[8];
uint8_t can_send_buf[8] = {1,2,3,4,5,6,7,8};

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    rxbndlc_t rxbndlc0, rxbndlc1;
    rx_status_t rx_status;
    read_rx_id_t rx_id0, rx_id1;

    static uint8_t rtr_cnt = 0;

    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);

    printf("SystemClk:%d\r\n", SystemCoreClock);

    printf("CH9431 SEND RECV Test\r\n");

    SPI_FullDuplex_Init();

    NVIC_DisableIRQ(EXTI15_10_IRQn);
    CH9431_Init();
    ch9431_int_sign = 0;
    ch9431_rx0_sign = 0;
    ch9431_rx1_sign = 0;
    NVIC_EnableIRQ(EXTI15_10_IRQn);

    do
    {
        rxipctrl_t rxipctrl = {.B0BFE = 1, .B0BFM = 1, .B1BFE = 1, .B1BFM = 1, .B0BFS = 0, .B1BFS = 0};
        CH9431_WriteByte(R8_RXIPCTRL, rxipctrl.value);
        PRINTF("rxipctrl set ok\n");
    } while (0);    /* Ϊж */

    while(1)
    {
        do
         {
             CH9431_Read_RX_STATUS(&rx_status);
             PRINTF("rx_status:%02x\n", rx_status.value);
         } while (0);

         if(rx_status.RX0I != 0)
         {
             if(CH9431_Receive_Buffer0(can_rec_buf0, &rxbndlc0) != 0)
             {
                 PRINTF("rec 0, len: %d\n", rxbndlc0.DLC);

                 CH9431_Read_RX_ID(CH9431_ReadRX_ID0, &rx_id0);
                 PRINTF("SID: %02x\n", rx_id0.rxbn.SID);
                 PRINTF("EID: %02x\n", rx_id0.rxbn.EID);
                 PRINTF("IDE: %02x\n", rx_id0.rxbn.IDE);
                 PRINTF("DLC: %02x\n", rx_id0.rxbndlc.DLC);
                 PRINTF("RTR: %02x\n", rx_id0.rxbndlc.RTR);

                 if(rx_id0.rxbndlc.RTR == 0)
                 {
                     if(rxbndlc0.DLC > 0)
                     {
                         PRINTF("data:");
                         for(uint8_t i = 0; i < rxbndlc0.DLC; i++)
                         {
                             PRINTF(" %02x", can_rec_buf0[i]);
                         }
                         PRINTF("\n");
                     }
                 }
                 else
                 {
                     PRINTF("\n");
                 }
                 CH9431_BIT_Modify(R8_SYSINTF, 0x01, 0);                     /* ʹλָжϣֹжϷ */
             }
             else
             {
                 PRINTF("CH9431_Receive_Buffer0 failed\n");
             }
         }

         if(rx_status.RX1I != 0)
         {
             if(CH9431_Receive_Buffer1(can_rec_buf1, &rxbndlc1) != 0)
             {
                 PRINTF("rec 1, len: %d\n", rxbndlc1.DLC);

                 CH9431_Read_RX_ID(CH9431_ReadRX_ID1, &rx_id1);
                 PRINTF("SID: %02x\n", rx_id1.rxbn.SID);
                 PRINTF("EID: %02x\n", rx_id1.rxbn.EID);
                 PRINTF("IDE: %02x\n", rx_id1.rxbn.IDE);
                 PRINTF("DLC: %02x\n", rx_id1.rxbndlc.DLC);
                 PRINTF("RTR: %02x\n", rx_id1.rxbndlc.RTR);

                 if(rx_id1.rxbndlc.RTR == 0)
                 {
                     if(rxbndlc1.DLC > 0)
                     {
                         PRINTF("data:");
                         for(uint8_t i = 0; i < rxbndlc1.DLC; i++)
                         {
                             PRINTF(" %02x", can_rec_buf1[i]);
                         }
                         PRINTF("\n");
                     }
                 }
                 else
                 {
                     PRINTF("\n");
                 }
                 CH9431_BIT_Modify(R8_SYSINTF, 0x02, 0);                     /* ʹλָжϣֹжϷ */
             }
             else
             {
                 PRINTF("CH9431_Receive_Buffer1 failed\n");
             }
         }


         _port_ch9431_delay_ms(500);    /* ӳ0.5s */

         if(rtr_cnt == 0)
         {
             CH9431_Send_Buffer0(can_send_buf, 8, rtr_cnt);
         }
         else
         {
             CH9431_Send_Buffer0(can_send_buf, 8, rtr_cnt);    /* Զ֡޸ĳȲ */
         }
         rtr_cnt = 1 - rtr_cnt;  /* ֻԶ֡ϴԶ֡֡ */
         if(rtr_cnt == 0)
         {
             CH9431_Send_Buffer1(can_send_buf, 7, rtr_cnt);
         }
         else
         {
             CH9431_Send_Buffer1(can_send_buf, 7, rtr_cnt);    /* Զ֡޸ĳȲ */
         }
         rtr_cnt = 1 - rtr_cnt;
         if(rtr_cnt == 0)
         {
             CH9431_Send_Buffer2(can_send_buf, 6, rtr_cnt);
         }
         else
         {
             CH9431_Send_Buffer2(can_send_buf, 6, rtr_cnt);    /* Զ֡޸ĳȲ */
         }
         rtr_cnt = 1 - rtr_cnt;

         for(uint8_t i = 0; i < 8; i++)
         {
             can_send_buf[i]++;
         }

         _port_ch9431_delay_ms(1000);        /* ӳ1s */

         do
         {
             eflag_t eflag;
             eflag.value = CH9431_ReadByte(R8_EFLAG);
             PRINTF("eflag:%02x\n", eflag.value);
         } while (0);

         do
         {
             uint8_t err_rec, err_tec;

             err_rec = CH9431_ReadByte(R8_REC);
             PRINTF("err_rec:%02x\n", err_rec);

             err_tec = CH9431_ReadByte(R8_TEC);
             PRINTF("err_tec:%02x\n", err_tec);
         } while (0);
    }
}
