/********************************** (C) COPYRIGHT *******************************
 * File Name          : ch9431_port.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/10/10
 * Description        : This file contains ch9431 port to ch32v20x.
*********************************************************************************
* Copyright (c) 2023 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "ch9431_port.h"

/**
 * @brief   delay t microseconds.
 *
 * @param   t - the num of us need to delay.
 *
 * @return  None.
*/
void _port_ch9431_delay_us(uint32_t t)
{
    Delay_Us(t);            /* 延迟us */
}

/**
 * @brief   delay t milliseconds.
 *
 * @param   t - the num of ms need to delay.
 *
 * @return  None.
*/
void _port_ch9431_delay_ms(uint32_t t)
{
    Delay_Ms(t);            /* 延迟ms */
}

/**
 * @brief   set rst pin to high.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_rst_high(void)
{
    GPIO_SetBits(GPIOB, GPIO_Pin_2);        /* 失能复位 */
}

/**
 * @brief   set rst pin to low.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_rst_low(void)
{
    GPIO_ResetBits(GPIOB, GPIO_Pin_2);      /* 使能复位 */
}

/**
 * @brief   set nss pin to high.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_spi_nss_high(void)
{
    GPIO_SetBits(GPIOA, GPIO_Pin_4);        /* 失能片选 */
}

/**
 * @brief   set nss pin to low.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_spi_nss_low(void)
{
    _port_ch9431_delay_us(CH9431_CMD_CMD_US);   /* 保证两条指令直接的命令间隔 */
    GPIO_ResetBits(GPIOA, GPIO_Pin_4);          /* 使能片选 */
}

/**
 * @brief   set tx0rts pin to low.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx0rts_low(void)
{
    GPIO_ResetBits(GPIOB, GPIO_Pin_6);
}

/**
 * @brief   set tx0rts pin to high.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx0rts_high(void)
{
    GPIO_SetBits(GPIOB, GPIO_Pin_6);
}

/**
 * @brief   set tx1rts pin to low.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx1rts_low(void)
{
    GPIO_ResetBits(GPIOB, GPIO_Pin_7);
}

/**
 * @brief   set tx1rts pin to high.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx1rts_high(void)
{
    GPIO_SetBits(GPIOB, GPIO_Pin_7);
}

/**
 * @brief   set tx2rts pin to low.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx2rts_low(void)
{
    GPIO_ResetBits(GPIOB, GPIO_Pin_8);
}

/**
 * @brief   set tx2rts pin to high.
 *
 * @param   None.
 *
 * @return  None.
*/
void _port_ch9431_tx2rts_high(void)
{
    GPIO_SetBits(GPIOB, GPIO_Pin_8);
}

/**
 * @brief   write and read a 16bit or 8bit data with spi.
 *
 * @param   data - data need to send.
 *
 * @return  data received of uint16_t.
*/
uint16_t _port_ch9431_spi_rw_data(uint16_t data)
{
    uint16_t rec;
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    SPI_I2S_SendData(SPI1, data);
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
    rec = SPI_I2S_ReceiveData(SPI1);
    return rec;
}
